#ifndef BOOST_GRAPH_H
#define BOOST_GRAPH_H

#include "boost/graph/adjacency_list.hpp"
#include "ViennaFile.h"
#include <string>
using namespace std;
using namespace boost;

//=========================================================================================================================

// Vertex and Edge property structs. Used as bundled properties

struct VertexProperty
{
	string id;
	int index;
};

struct EdgeProperty
{
	string id;
	int weight;
};

//=========================================================================================================================

// Typedefinitions

// Out-edge list type: std::list (1st parameter)
// Vertex list type  : std::list (2nd parameter)
// Edge list type    : std::list (7th parameter)

// Undirected graph
typedef adjacency_list<listS, vecS, undirectedS, VertexProperty, EdgeProperty, no_property, listS> UndirectedGraph;

// Vertex and edge descriptors
typedef graph_traits<UndirectedGraph>::vertex_descriptor VertexDescriptor;
typedef graph_traits<UndirectedGraph>::edge_descriptor EdgeDescriptor;

// Iterators
typedef graph_traits<UndirectedGraph>::vertex_iterator VertexIterator;
typedef graph_traits<UndirectedGraph>::edge_iterator EdgeIterator;
typedef graph_traits<UndirectedGraph>::adjacency_iterator AdjacencyIterator;
typedef graph_traits<UndirectedGraph>::out_edge_iterator OutEdgeIterator;

//=========================================================================================================================

class BoostGraph
{
	public:
		BoostGraph();
		~BoostGraph();

		bool LoadVienna(ViennaFile& viennaFile, bool displayDebugText); // Implementated!
		bool LoadSIF(vector<vector<string>>& data, bool displayDebugText); // (Implemented!)
		bool ExportToCytoscape(const string& filePath); // Implementated!
		void Print(); // Implemented!
		void PrintVertices();

		void DijkstraAllPairsShortestPaths(); // Lacks implementation
		void JohnsonsAllPairsShortestPaths(); // Lacks implementation
		bool Compare(BoostGraph& comparableGraph); // Lacks implementation
		bool AddVertex(string base, int position, bool debug);
		bool RemoveVertex(int position, bool debug);
		bool AddEdge(string base1, string base2, int position1, int position2, bool debug);
		bool RemoveEdge(int position1, int position2, bool debug);
		bool Change(int position, string base, bool debug);

		int GetNoVertices(); // Implemented!
		int GetNoEdges(); // Implemented!
		int GetNetworkDiameter(); // Lacks implementation
		bool IsEmpty(); // Implemented!

		UndirectedGraph& GetGraph();

		ViennaFile sourceViennaFile;

	private:
		VertexDescriptor AddVertex(string id);

	private:
		UndirectedGraph uGraph;
};

//=========================================================================================================================

#endif